<?php

namespace Drupal\graphql_flag\Plugin\GraphQL\Fields;

use Drupal\flag\FlagInterface;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use GraphQL\Type\Definition\ResolveInfo;

/**
 * Provides the flagged entities from a Flag entity id.
 *
 * Limited to the current authenticated user.
 *
 * @GraphQLField(
 *   id = "user_flagging",
 *   name = "userFlagging",
 *   description = @Translation("List of flagged entities from a User."),
 *   secure = true,
 *   type = "UserFlaggingList",
 *   parents = {
 *     "User",
 *   },
 *   arguments = {
 *     "flag_id" = "[String!]"
 *   },
 *   response_cache_max_age = 0,
 * )
 */
class UserFlagging extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    $currentUser = \Drupal::currentUser();
    // @todo add session support for anonymous user.
    // @todo review if some parts could be delegated to the flag service.
    // @todo review user cache context.
    // @todo dependency injection.
    if (!$currentUser->isAnonymous()) {
      $flagIds = $args['flag_id'];
      /** @var \Drupal\flag\FlagServiceInterface $flagService */
      $flagService = \Drupal::service('flag');
      $userFlags = [];
      foreach ($flagIds as $flagId) {
        $flag = $flagService->getFlagById($flagId);
        if ($flag instanceof FlagInterface) {
          $query = \Drupal::database()
            ->select('flagging', 'f')
            ->fields('f', ['entity_type', 'entity_id'])
            ->condition('flag_id', $flag->id())
            ->condition('uid', \Drupal::currentUser()->id());
          $queryResult = $query->execute();
          foreach ($queryResult as $userFlag) {
            $userFlags[] = [
              'flagId' => $flag->id(),
              'entityType' => $userFlag->entity_type,
              'entityId' => $userFlag->entity_id,
            ];
          }
        }
      }
      yield [
        'user_flaggings' => $userFlags,
      ];
    }
  }

}
